﻿/*------------------------------------------------*/
/* 26regexp/pcre.c                                */

#include "parse-options-pcre.h"
#include <pcre.h>
#include <string.h>
#include <locale.h>

#define N 50

void printRegerr(int ret);
static void printPatterns(int ret, int pmatch[], char * text);

int main(int argc, char *argv[]) {
    pcre *regex;
    pcre_extra *study;
    task t;
    int ret, options;
    int pmatch[N];
    int erroffset;
    const char *errptr;

    setlocale(LC_ALL,"");

    t = parseOptions(argc, argv);
    if(t.help || t.err) { 
        printOptionsHelp(); 
        return 1; 
    }
    if(t.show) {
        printOptions(t);
    }
    printf("\n");

    options = PCRE_NEWLINE_ANYCRLF | PCRE_UTF8;
    if(t.icase) 
        options |= PCRE_CASELESS;
    if(t.newline)
        options |= PCRE_MULTILINE;
    if(t.ucp)
        options |= PCRE_UCP;

    /* Kompilace reg. výrazu */
    regex = pcre_compile(t.pattern, options, &errptr, &erroffset, NULL);

    if(regex == NULL) {
        fprintf(stderr, "Nepodařilo se zkompilovat reg. exp. '%s'\n\n%s\n",
            t.pattern, errptr);
        return 1;
    }

    /* Optimalizace */
    study = pcre_study(regex, 0, &errptr);

   /* pcre_study() vrací NULL při chybě, ale i když jen nemůže
    * optimalizovat regex. Rozdíl se pozná podle pcreErrorStr */
    if(errptr != NULL) {
        printf("Chyba optimalizace: %s\n", errptr);
        return 1;
    }

    /* Vyhledávání reg. výrazů */
    ret = pcre_exec(regex, study, 
                   t.string,         /* prohledávaný text */
                   strlen(t.string), /* délka prohledávaného textu */
                   0,                /* Začni prohledávat od tohoto indexu */
                   0,                /* Další options, viz manuálové stránky */
                   pmatch,
                   sizeof(pmatch)/sizeof(pmatch[0])
    );

    /* Vypiš chybu */
    if (ret >= 0) {
        puts("PATTERN BYL NALEZEN.");
    /* vypsání nalezených částí reg. výrazu */
        printPatterns(ret, pmatch, t.string);
    }
    else if(ret == PCRE_ERROR_NOMATCH) {
        puts("PATTERN nebyl NALEZEN.");
    } else {
        puts("CHYBA");
        printRegerr(ret);
    return 1;
    } 
      
    pcre_free(regex);
    if(study != NULL) {
        pcre_free_study(study);
    }
    printf("\n");
      
    return 0;
}

/**
 * Vypíše chybu
 */
void printRegerr(int ret) {
    switch(ret) {
        case PCRE_ERROR_NULL           : fprintf(stderr,"Regex nebo t.string byl NULL\n");            break;
        case PCRE_ERROR_BADOPTION      : fprintf(stderr,"V options byl nastaven neznámý bit\n");      break;
        case PCRE_ERROR_BADMAGIC       : fprintf(stderr,"Zkopilovaný regex je vadný\n");              break;
        case PCRE_ERROR_UNKNOWN_OPCODE : fprintf(stderr,"Bug v PCRE, nebo poškozený regex");          break;
        case PCRE_ERROR_NOMEMORY       : fprintf(stderr,"Nedostatek paměti\n");                       break;
        case PCRE_ERROR_MATCHLIMIT     : fprintf(stderr,"Byl překročen match_limit\n");               break;
        case PCRE_ERROR_RECURSIONLIMIT : fprintf(stderr,"Byl překročen match_limit_recursion\n");     break;
        case PCRE_ERROR_SHORTUTF8      :
        case PCRE_ERROR_BADUTF8_OFFSET :
        case PCRE_ERROR_BADUTF8        : fprintf(stderr,"Chybný UTF-8 znak\n");                       break;
        case PCRE_ERROR_BADOFFSET      : fprintf(stderr,"Offset byl negativní, nebo větší než délka t.string\n"); break;
        case PCRE_ERROR_RECURSELOOP    : fprintf(stderr,"V regexu byla detekována nekonečná rekurze\n"); break;
        case PCRE_ERROR_BADLENGTH      : fprintf(stderr,"Zadaná délka je negativní\n");               break;
        default                        : fprintf(stderr,"Neznámá chyba\n");                           break;
    }
}

/**
 * Vypíše nalezené části textu
 */
static void printPatterns(int ret, int pmatch[], char * text) {
    int j;
    const char *stringptr;
    if(ret == 0) {
       printf("Bylo nalezeno příliš mnoho částí reg. výrazu!\n");
       ret = N / 2;
    }

    for(j=0; j< ret*2; j+=2) {
        pcre_get_substring(text, pmatch, ret, j/2, &stringptr);
        printf("Match: (%2i,%2i): %i '%s'\n", pmatch[j], pmatch[j+1], pmatch[j+1]-pmatch[j], stringptr);
        pcre_free_substring(stringptr);
    }
}

/*------------------------------------------------*/
